<?php

namespace Drupal\Tests\commerce_exchanger_cryptocompare\Kernel;

use Drupal\commerce_exchanger\Entity\ExchangeRates;
use Drupal\commerce_price\Entity\Currency;
use Drupal\commerce_price\Price;
use Drupal\Tests\commerce\Kernel\CommerceKernelTestBase;

/**
 * Kernel tests to ensure modules functionality.
 *
 * @group commerce_exchanger_cryptocompare
 */
class CurrencyCodeMappingTest extends CommerceKernelTestBase {

  /**
   * Price in BTC currency.
   *
   * @var \Drupal\commerce_price\Price
   */
  protected $priceLUNA;

  /**
   * Price in ETH currency.
   *
   * @var \Drupal\commerce_price\Price
   */
  protected $priceTRTL;

  /**
   * Imported exchange rates for BTC.
   *
   * @var array
   */
  protected $ratesLUNA;

  /**
   * Imported exchange rates for ETH.
   *
   * @var array
   */
  protected $ratesTRTL;

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'commerce_exchanger',
    'commerce_exchanger_cryptocompare',
    'commerce_exchanger_cryptocompare_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->installConfig(['commerce_exchanger']);
    $this->installConfig(['commerce_exchanger_cryptocompare_test']);

    // Add third party setting currency_code_mapping.
    $crypto_compare_exchanger = $this->entityTypeManager->getStorage('commerce_exchange_rates')->load('cryptocompare');
    $crypto_compare_exchanger->setThirdPartySetting('commerce_exchanger_cryptocompare', 'currency_code_mapping', 'TRT|TRTL
LUN|LUNA')
      ->save();

    // Add LUNA and TRTL currencies.
    $luna = Currency::create([
      'currencyCode' => 'LUN',
      'name' => 'Terra',
      'numericCode' => '000',
      'symbol' => 'LUNA',
      'fractionDigits' => 2,
    ]);
    $luna->save();

    $turtlecoin = Currency::create([
      'currencyCode' => 'TRT',
      'name' => 'TurtleCoin',
      'numericCode' => '000',
      'symbol' => 'TRTL',
      'fractionDigits' => 2,
    ]);
    $turtlecoin->save();

    $crypto_compare_exchange_rates = ExchangeRates::load('cryptocompare')->getPlugin();
    $crypto_compare_exchange_rates->import();

    $this->ratesLUNA = $crypto_compare_exchange_rates->getRemoteData('LUN');
    $this->ratesTRTL = $crypto_compare_exchange_rates->getRemoteData('TRT');

    $this->priceLUNA = new Price('1', 'LUN');
    $this->priceTRTL = new Price('1', 'TRT');
  }

  /**
   * Test exchanger plugin import functionality.
   */
  public function testExchangerRatePluginImport() {
    $exchange_rates = $this->container->get('commerce_exchanger.calculate')->getExchangeRates();

    $this->assertArrayHasKey('LUN', $exchange_rates, 'LUNA rates not available');
    $this->assertNotNull($exchange_rates["USD"]["LUN"]["value"], 'No LUNA - USD rate imported.');

    $this->assertArrayHasKey('TRT', $exchange_rates, 'TRTL rates not available');
    $this->assertNotNull($exchange_rates["USD"]["TRT"]["value"], 'No TRTL - USD rate imported.');
  }

  /**
   * Test actual BTC price conversion.
   */
  public function testLunaExchangerRatesPluginRemoteData() {
    $this->assertArrayHasKey('rates', $this->ratesLUNA, 'No LUNA rates present');
    $this->assertArrayHasKey('LUN', $this->ratesLUNA['rates'], 'No LUNA/USD rate present');

    $priceUsd = $this->ratesLUNA['rates']['USD'];
    $this->assertNotNull($priceUsd, 'No LUNA - USD rate imported.');
    $this->assertEqualsWithDelta(50.0, $priceUsd, 20.0, 'Either test failed or LUNA is up/down by a lot!');
  }

  /**
   * Test actual ETH price conversion.
   */
  public function testTrtlExchangerRatesPluginRemoteData() {
    $this->assertArrayHasKey('rates', $this->ratesTRTL, 'No TRTL rates present');
    $this->assertArrayHasKey('USD', $this->ratesTRTL['rates'], 'No TRTL/USD rate present');

    $priceUsd = $this->ratesTRTL['rates']['USD'];
    $this->assertNotNull($priceUsd, 'No TRTL - USD rate imported.');
    $this->assertEqualsWithDelta(0.0001, $priceUsd, 0.0002, 'Either test failed or TRTL is up/down by a lot!');
  }

}
